/// <reference types="node" />
import { Device } from "./device";
import { IDeviceInterface } from "../interfaces";
/**
 * @class TechnicMediumHubTiltSensor
 * @extends Device
 */
export declare class TechnicMediumHubTiltSensor extends Device {
    protected _impactThreshold: number;
    protected _impactHoldoff: number;
    constructor(hub: IDeviceInterface, portId: number);
    receive(message: Buffer): void;
    /**
     * Set the impact count value.
     * @method TechnicMediumHubTiltSensor#setImpactCount
     * @param {count} impact count between 0 and 2^32
     * @returns {Promise} Resolved upon successful issuance of the command.
     */
    setImpactCount(count: number): Promise<void>;
    /**
     * Set the impact threshold.
     * @method TechnicMediumHubTiltSensor#setImpactThreshold
     * @param {threshold} value between 1 and 127
     * @returns {Promise} Resolved upon successful issuance of the command.
     */
    setImpactThreshold(threshold: number): Promise<void>;
    /**
     * Set the impact holdoff time.
     * @method TechnicMediumHubTiltSensor#setImpactHoldoff
     * @param {holdoff} value between 1 and 127
     * @returns {Promise} Resolved upon successful issuance of the command.
     */
    setImpactHoldoff(holdoff: number): Promise<void>;
}
export declare enum Mode {
    TILT = 0,
    IMPACT_COUNT = 1
}
export declare const ModeMap: {
    [event: string]: number;
};
